
* EXAMPLE 1;
data deposits;
input account :$2. month deposit @@;	
datalines;
01 2 100 01 4 50  01 6 200
02 1 50  02 3 100
03 1 50  03 2 50  03 3 50 03 4 50  03 5 50  03 6 50
;

title "DATA SET DEPOSITS";
proc print data=deposits;
run;


* EXAMPLE 2;
* SAS-supplied name for new data set;
proc transpose data=deposits;
run;

* user-supplied name for new data set;
proc transpose data=deposits out=accounts;
run;

title "DATA SET ACCOUNTS - TRANSPOSE WITH NO OPTIONS SPECIFIED";
proc print data=accounts;
run;


* EXAMPLE 3;
proc transpose data=deposits out=accounts prefix=dep;
by account;
run;

title "DATA SET ACCOUNTS - ADD A PREFIX";
proc print data=accounts;
run;


* EXAMPLE 4;
proc transpose data=deposits out=accounts (drop=_name_) prefix=dep;
by account;
var deposit;
id month;
run;

title "DATA SET ACCOUNTS - ADD AN ID VARIABLE";
proc print data=accounts;
run;

title "DATA SET ACCOUNTS - PRINT DEPOSITS IN MONTH ORDER";
proc print data=accounts;
var account dep1-dep6;
run;


* find diffferece between first and last deposit;
* find the months that those deposits occurred;
data first_last;
set accounts;
first_dep = coalesce(of dep1-dep6);
last_dep = coalesce(of dep6-dep1);
first_month = whichn(first_dep, of dep1-dep6);
last_month = 7 - whichn(last_dep,of dep6-dep1);
run;

title "DATA SET ACCOUNTS - WITH FIRST AND LAST INFORMATION ADDED";
proc print data=first_last;
id account dep1-dep6;
run;


* leave out PREFIX;
proc transpose data=deposits out=accounts (drop=_name_);
by account;
var deposit;
id month;
run;

title 'ACCOUNTS ... NO PREFIX SPECIFIED';
proc print data=accounts;
run;

*****************************************************************************;

data depositsx;
input account :$2. month :$3. deposit @@;	
datalines;
01 FEB 100 01 APR 50  01 JUN 200
02 JAN 50  02 MAR 100
03 JAN 50  03 FEB 50  03 MAR 50 03 APR 50  03 MAY 50  03 JUN 50
;

proc transpose data=depositsx out=accounts (drop=_name_);
by account;
var deposit;
id month;
run;

title "DATA SET ACCOUNTS - MONTH IS CHARACTER VARIABLE";
proc print data=accounts;
run;

* replace character month with numeric month;
data depositsx;
set depositsx (rename=(month=temp));
month = month(input(catt("01",temp,"00"),date7.));
run;

title "DATA SET ACCOUNTS - MONTH IS CHARACTER VARIABLE";
proc print data=depositsx;
run;

*****************************************************************************;

* new data set with additional variable TYPE;
data deposits;
input account :$2. month deposit type @@;	
datalines;
01 2 100 1 01 4 50 1 01 6 200 2
02 1 50 2 02 3 100 2
03 1 50 1 03 2 50 1 03 3 50 2 03 4 50 1 03 5 50 1 03 6 50 1
;

title "DATA SET DEPOSITS";
proc print data=deposits;
run;

* EXAMPLE 5;
proc transpose data=deposits out=accounts prefix=month;
by account;
var deposit type;
id month ;
run;

title "DATA SET ACCOUNTS - TWO TRANSPOSED VARIABLES";
proc print data=accounts;
run;

* EXAMPLE 6;
data new_deposits (keep=account new1 new2 month);
set deposits;
new1 = 'D' ; new2 = deposit; output;
new1 = 'T' ; new2 = type; output;
run;

title "DATA SET NEW_DEPOSITS - DEPOSITS REARRANGED FOR TRANSPOSE";
proc print data=new_deposits;
run;

* EXAMPLE 7;
proc transpose data=new_deposits out=accounts (drop=_name_);
by account;
var new2;
id new1 month;
run;

title "DATA SET ACCOUNTS - TRANSPOSE WITH TWO ID VARIABLES";
proc print data=accounts;
var account d: t: ;
run;

*****************************************************************************;

* recreate original data set;
data deposits;
input account :$2. month deposit @@;	
datalines;
01 2 100 01 4 50  01 6 200
02 1 50  02 3 100
03 1 50  03 2 50  03 3 50 03 4 50  03 5 50  03 6 50
;

title "DATA SET DEPOSITS";
proc print data=deposits;
run;

* EXAMPLE 8;
data accounts (keep=account dep1-dep6);
array dep(6);
do until (last.account);
   set deposits;
   by account;
   dep(month) = deposit;
end;
run;

title "DATA SET ACCOUNTS CREATED WITH A DATA STEP AND AN ARRAY";
proc print data=accounts;
var account dep: ;
run;


* EXAMPLE 9;
data accounts (keep=account dep1-dep6);
array dep(6);
do _n_ = 1 by 1 until (last.account);
   set deposits;
   by account;
   dep(_n_) = deposit;
end;
run;

title "DATA SET ACCOUNTS CREATED WITH A DATA STEP AND AN ARRAY (MONTH NOT USED)";
proc print data=accounts;
var account dep: ;
run;

* new data set with additional variable TYPE;
data deposits;
input account :$2. month deposit type @@;	
datalines;
01 2 100 1 01 4 50 1 01 6 200 2
02 1 50 2 02 3 100 2
03 1 50 1 03 2 50 1 03 3 50 2 03 4 50 1 03 5 50 1 03 6 50 1
;

title "DATA SET DEPOSITS";
proc print data=deposits;
run;


* EXAMPLE 10;
proc sql noprint;
select max(month) into :mm trimmed from deposits;
quit;

%put |&mm|;

data accounts (keep=account amt1-amt&mm typ1-typ&mm);
array amt(&mm);
array typ(&mm);
do until (last.account);
   set deposits;
   by account;
   amt(month) = deposit;
   typ(month) = type;
end;
run;

title "DATA SET ACCOUNTS CREATED WITH A DATA STEP, ARRAYS, AND A MACRO VARIABLE";
proc print data=accounts;
var account amt: typ: ;
run;

* new data set with additional variable TYPE but no MONTH;
data deposits;
input account :$2. deposit type @@;	
datalines;
01 100 1 01 50 1 01 200 2
02 50 2 02 100 1
03 25 1 03 50 2 03 50 1 03 100 2 03 50 1
;

title "DATA SET DEPOSITS";
proc print data=deposits;
run;


* EXAMPLE 11;
* find maximum number of depoits within an account;
proc sql noprint;
select max(count) into :mm separated by '' from
(select count(*) as count from deposits group by account);
quit;

%put &mm;

data accounts (keep=account amt1-amt&mm typ1-typ&mm);
array amt(&mm);
array typ(&mm);
do j = 1 by 1 until (last.account);
   set deposits;
   by account;
   amt(j) = deposit;
   typ(j) = type;
end;
run;

title "DATA SET ACCOUNTS CREATED WITH A DATA STEP, ARRAYS, AND A MACRO VARIABLE";
proc print data=accounts;
var account amt: typ: ;
run;

*****************************************************************************;

* new data set with additional variable TYPE but no MONTH;
data deposits;
input account :$2. deposit type @@;	
datalines;
01 100 1
02 50  2
03 25 1
01 50 .
02 100 1
03 . 2
01 200 2
03 50 .
03 100 2
;

title "DATA SET DEPOSITS";
proc print data=deposits;
run;


* EXAMPLE 12;
* one variable;
proc sql noprint;
select max(count) into :mm separated by '' from
(select count(*) as count from deposits group by account);
quit;

proc summary data=deposits nway;
class account;
output out=accounts (drop=_type_ _freq_) idgroup(out[&mm](deposit)=dep);
run;

title "DATA SET ACCOUNTS CREATED USING PROC SUMMARY AND IDGROUP";
proc print data=accounts;
run;


* EXAMPLE 13;
* two variables;
proc sql noprint;
select max(count) into :mm separated by '' from
(select count(*) as count from deposits group by account);
quit;

proc summary data=deposits nway;
class account;
output out=accounts (drop=_type_ _freq_) idgroup(out[&mm](deposit type)=dep typ);
run;

title "DATA SET ACCOUNTS CREATED USING PROC SUMMARY AND IDGROUP - TWO VARIABLES";
proc print data=accounts;
run;


* EXAMPLE 14;
* multiple idgroups;
data deposits;
input account :$2. month deposit @@;	
datalines;
01 2 100 01 4 50  01 6 200
02 1 50  02 3 100
03 1 50  03 2 50  03 3 50 03 4 50  03 5 50  03 6 50
;

title "DATA SET DEPOSITS";
proc print data=deposits;
run;

proc summary data=deposits nway;
class account;
output out=accounts (drop=_:)
       idgroup(min(month) out(deposit)=dep_first)
       idgroup(max(month) out(deposit)=dep_last);
run;

title "DATA SET ACCOUNTS - FIRST AND LAST DEPOSIT BASED ON VALUE OF MONTH";
proc print data=accounts;
run;

* EXAMPLE 15;
* new data set with additional variable TYPE;
* use AUTONAME;

data deposits;
input account :$2. month deposit type @@;	
datalines;
01 2 100 1 01 4 50 1 01 6 200 2
02 1 50 2 02 3 100 2
03 1 50 1 03 2 50 1 03 3 50 2 03 4 50 1 03 5 50 1 03 6 50 1
;

proc summary data=deposits nway;
class account;
output out=accounts (drop=_:)
      idgroup(min(month) out(deposit type)=)
      idgroup(max(month) out(deposit type)=) / autoname ;

run;

title "DATA SET ACCOUNTS - FIRST AND LAST DEPOSIT BASED ON VALUE OF MONTH / AUTONAME";
proc print data=accounts;
run;


* EXAMPLE 16;
libname x 'd:\epi514\datasets';


data clinical;
do j=1 by 1 until (last.id);
   set x.clinical;
   by id;
   if j le 5 then output;
end;
drop j;
run;

title "DATA SET CLINICAL";
proc print data=clinical;
where ranuni(0) le 0.2;
run;

* chol, sbp, and dbp at first and last visits slected by observation location within an id;
proc summary data=clinical nway;
class id;
output out=clin_first_last (drop=_:)
      idgroup(out(chol sbp dbp)=)
      idgroup(last out(chol sbp dbp)=) / autoname ;
run;

title "CHOLESTEROL AND BLOOD PRESSURES ... FIRST AND LAST WITHIN AN ID";
proc print data=clin_first_last;
run;

* chol, sbp, and dbp at first two and last two visits selected using visit dates;
proc summary data=x.clinical nway ;
class id;
output out=clin_first_last (drop=_type_)
      idgroup(min(visit) out [2] (visit sbp dbp)=)
      idgroup(max(visit) out [2] (visit sbp dbp)=) / autoname ;
run;

title "CHOLESTEROL AND BLOOD PRESSURES ... FIRST AND LAST TWO WITHIN AN ID BASED ON DATE";
proc print data=clin_first_last nobs;
run;

* what do min and max mean with more than one variable;
* chol, sbp, and dbp at min/max combination sbp and dbp;
proc summary data=x.clinical nway;
class id;
output out=clin_first_last (drop=_:)
      idgroup(min(sbp dbp) out (sbp dbp chol hr)=)
      idgroup(max(sbp dbp) out (sbp dbp chol hr)=) / autoname ;
run;

title "MULTIPLE MIN AND MAX VARIABLES";
proc print data=clin_first_last;
run;

proc sort data=x.clinical out=check;
by id sbp dbp;
where id in ('01' '07');
run;

proc print data=check;
by id;
run;

*****************************************************************************;

* keep _FREQ_;
proc summary data=x.clinical nway;
class id;
output out=clin_first_last (drop=_type_ rename=(_freq_=visits))
      idgroup(min(chol) out(chol)=min_chol)
      idgroup(max(chol) out(chol)=max_chol) ;

run;

title;
proc print data=clin_first_last;
run;

*****************************************************************************;
* APPENDIX A;

libname x 'd:\epi514\datasets';

* use five observations from each id;
data clinical;
do j=1 by 1 until (last.id);
   set x.clinical;
   by id;
   if j le 5 then output;
end;
run;

proc print data=clinical (obs=10);
var id chol sbp dbp hr;
run;

data new_clinical (keep=id new:);
array x(4) chol sbp dbp hr;
do new1 = 1 by 1 until(last.id);
   set clinical;
   by id;
do _n_ = 1 to 4;
   new2 = x(_n_);
   new3 = vname(x(_n_));
   output;
end;
end;
run;

proc print data=new_clinical;
run;

proc transpose data=new_clinical out=patients (drop=_name_);
by id;
var new2;
id new3 new1;
run;

proc print data=patients;
var id chol: sbp: dbp: hr: ;
run;


proc sql noprint;
select max(count) into :mm separated by '' from
(select count(*) as count from clinical group by id);
quit;

proc summary data=clinical nway;
class id;
output out=new_patients (drop=_type_ _freq_) idgroup(out[&mm](chol sbp dbp hr)=);
run;

proc print data=new_patients;
var id chol: sbp: dbp: hr: ;
run;


* use all the data;
proc sql noprint;
select max(count) into :mm separated by '' from
(select count(*) as count from x.clinical group by id);
quit;

proc summary data=x.clinical nway;
class id;
output out=new_patients (drop=_type_ _freq_) idgroup(out[&mm](chol sbp dbp hr)=);
run;

proc print data=new_patients;
var id chol: sbp: dbp: hr: ;
run;
